module GraphQLPreview
  class SchemaModification
    DEFAULT_ENVIRONMENT = :all

    def self.environments(*new_environments)
      if new_environments.length > 0
        @environments = new_environments
      else
        @environments
      end
    end

    def self.to_h
      {
        title: title,
        description: description,
        toggled_by: toggled_by,
        announcement: announcement,
        updates: updates,
        toggled_on: toggled_on
      }
    end

    def self.title(new_title = nil)
      if new_title
        @title = new_title
      else
        @title
      end
    end

    def self.description(new_description = nil)
      if new_description
        @description = new_description
      else
        @description
      end
    end

    def self.toggled_by(new_toggled_by = nil)
      if new_toggled_by
        @toggled_by = new_toggled_by
      else
        @toggled_by
      end
    end

    def self.announced_on(date, url: nil)
      @announcement = {
        date: date,
        url: url
      }
    end

    def self.announcement
      @announcement
    end

    def self.updated_on(date, url: nil)
      @updates ||= []

      @updates <<  {
        date: date,
        url: url
      }
    end

    def self.updates
      @updates
    end

    def self.toggled_on(member_paths = nil)
      if member_paths
        member_paths.each do |path|
          unless path.is_a?(String)
            raise ArgumentError, "Provide string paths to `toggled_on`. For example, to toggle on a field called `name` on a type `Author`, use `\"Author.name\"``"
          end
        end

        @member_paths = member_paths
      else
        @member_paths
      end
    end

    def self.unset(schema)
      set_preview_toggled_by_on_members(schema, nil)
    end

    def self.apply_to_schema(schema, possible_environments)
      environments = if self.environments
        self.environments
      else
        possible_environments
      end

      value = Hash[environments.map{ |env| [env, self] }]

      set_preview_toggled_by_on_members(schema, value)
    end

    private

    def self.set_preview_toggled_by_on_members(schema, value)
      members = @member_paths.map { |path| schema.find(path) }

      members.each do |member|
        if member.is_a?(GraphQL::Field)
          # Let fields pass; we'll apply it during instrumentation
          next
        end

        unless member.respond_to?(:metadata)
          raise NoMethodError, "`#{member}` cannot have its metadata set"
        end

        # unless we're unsetting the preview, don't allow adding a new value
        if value && member.metadata[:preview_toggled_by]
          raise ArgumentError, "Couldn't add `#{self.toggled_by}` to `#{member.name}`: it's already toggled with `#{member.metadata[:preview_toggled_by]}`"
        end

        member.metadata[:preview_toggled_by] = value

        if member.is_a?(GraphQL::InterfaceType)
          member.fields.each_value do |field|
            field.metadata[:preview_toggled_by] = value
          end
        end
      end
    end
  end
end
